package com.springboot.jpa.auditing.controller;

import com.springboot.jpa.auditing.dto.ProductDto;
import com.springboot.jpa.auditing.dto.ProductResponse;
import com.springboot.jpa.auditing.entity.Product;
import com.springboot.jpa.auditing.exception.EntityNotFoundException;
import com.springboot.jpa.auditing.service.ProductService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.List;

//spring web annotations
@RestController
@RequestMapping("/api")
public class ProductController {

    @Autowired
    ProductService service;

    //get all products
    //URL - http://localhost:9800/api/products
    @GetMapping("/products")
    @ResponseStatus(code = HttpStatus.OK)
    public List<ProductResponse> getProducts() {
        return service.getAll();
    }

    //get product by id
    //URL - http://localhost:9800/api/product-by-id?id=1
    @GetMapping("/product-by-id")
    @ResponseStatus(code = HttpStatus.OK)
    public ProductResponse getProduct(@RequestParam(name = "id") final int id)
            throws EntityNotFoundException {
        final Product p = service.getProduct(id);
        return ProductResponse.from(p);
    }

    //update product by id
    //URL - http://localhost:9800/api/product-by-id?id=1
    //sample request body -
    /*
    {
        "productName":"Natraj Rubber",
        "material":"Rubber",
        "price":"9.21",
        "promotionCode":"IncrediblePrice1020"
    }
     */
    @PutMapping("/product-by-id")
    @ResponseStatus(code = HttpStatus.NO_CONTENT)
    public void updateProduct(@RequestParam(name = "id") final int id,
                              @RequestBody final ProductDto dto) throws EntityNotFoundException {
        service.updateProduct(id, dto);
    }
}
